<?php

namespace App\Http\Controllers\Api;

use App\Exceptions\FormException;
use App\Model\MediaModel;
use App\Model\QiniuModel;
use App\Tool\SrsHookValidate;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

use Qiniu\Auth;

class MediaController extends Controller
{

    /**
     * 获取媒资ID
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \App\Exceptions\ControllerException
     */
    public function media_list(Request $request){

        $result = handler_drive(function () use ($request){

            SrsHookValidate::srsHookCallback($request,[
                'params'    => 'required'
            ]);

            return MediaModel::getMediaList($request);

        });

        return success($result);
    }

    /**
     * 获取媒资播放地址
     * @param Request $request
     * @param $media_id
     * @return \Illuminate\Http\JsonResponse
     * @throws \App\Exceptions\ControllerException
     */
    public function play_url(Request $request,$media_id){

        $result = handler_drive(function () use ($request,$media_id){
            return MediaModel::playUrl($media_id);
        });

        return success($result);

    }



    /**
     * 获取媒资列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \App\Exceptions\ControllerException
     */
    public function mediaListOnce(Request $request){

        $res = handler_drive(function () use($request){

            SrsHookValidate::srsHookCallback($request,[
                'page'          => 'integer',
                'sort'          => '',
                'page_size'     => 'integer',
                'start_time'    => 'date',
                'end_time'      => 'date',
            ]);

            return MediaModel::mediaList($request);

        });
        return success($res);
    }


    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \App\Exceptions\ControllerException
     */
    public function handleMediaUrl(Request $request){

        $res = handler_drive(function () use($request){
            SrsHookValidate::srsHookCallback($request,[
                'media_name'    => '',
                'username'      => '',
                'page'          => 'integer',
                'media_id'      => '',
                'description'   => '',
                'sort'          => '',
                'page_size'     => 'integer',
            ]);
            return MediaModel::handleMediaUrl($request);
        });
        return success($res);
    }


    /**
     * 视频上传成功之后，回调重新上传
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \App\Exceptions\ControllerException
     */
    public function hookRestoreMedia(Request $request){


        $id = handler_drive(function () use ($request) {

            SrsHookValidate::srsHookCallback($request,array_merge([
                'media_name'  => 'required',
                'secret_key'  => 'required',
                'media_url'   => 'required',
                'description' => '',
            ]),function (...$params){

                $request = $params[0];

                if (!strstr($request->media_name,'.')){
                    throw new FormException('媒资名字格式有误');
                }

            });

            $data_time = date('Y-m-d H:i:s',time());

            $params = array(
                'media_id'  => 'mda-'.generateRandomNum(16),
                'media_type'=> '5efa9cb5d16f5a214671fd86',
                'start_time'=> $data_time,
                'end_time'  => $data_time,
                'access_key'=> '',
                'description'=>'网校',
                'create_time'=> $data_time,
            );

            $request->replace(array_merge($request->all(),$params));

            $id = MediaModel::hookRestoreMedia($request);

            return $id;
        });
        return success($id);
    }

    /**
     * 更新百度status
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \App\Exceptions\ControllerException
     */
    public static function updateMediaStatus(Request $request,$media_id){


        $id = handler_drive(function () use ($request,$media_id) {

            SrsHookValidate::srsHookCallback($request,[
                'status'    => ''
            ]);

            $id = MediaModel::updateMediaStatus($request,$media_id);

            return $id;
        });
        return success($id);

    }







}
